@@ -3,13 +3,17 @@
  *
  * Portions Copyright (c) 2008-2011, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  * Portions Copyright (c) 2011, Itagaki Takahiro
- * Portions Copyright (c) 2012-2015, The Reorg Development Team
+ * Portions Copyright (c) 2012-2020, The Reorg Development Team
  *-------------------------------------------------------------------------
  */
 
 #include "postgres_fe.h"
 #include "libpq/pqsignal.h"
 
+#if PG_VERSION_NUM >= 140000
+#include "common/string.h" /* for simple_prompt */
+#endif
+
 #include <limits.h>
 #include <sys/stat.h>
 #include <time.h>
@@ -450,7 +454,7 @@ prompt_for_password(void)
 		passwdbuf = pgut_malloc(BUFSIZE);
 		memcpy(passwdbuf, buf, sizeof(char)*BUFSIZE);
 	}
-#else
+#elif PG_VERSION_NUM < 140000
 	buf = pgut_malloc(BUFSIZE);
 	if (have_passwd) {
 		memcpy(buf, passwdbuf, sizeof(char)*BUFSIZE);
@@ -461,6 +465,16 @@ prompt_for_password(void)
 		passwdbuf = pgut_malloc(BUFSIZE);
 		memcpy(passwdbuf, buf, sizeof(char)*BUFSIZE);
 	}
+#else
+	if (have_passwd) {
+		buf = pgut_malloc(BUFSIZE);
+		memcpy(buf, passwdbuf, sizeof(char)*BUFSIZE);
+	} else {
+		buf = simple_prompt("Password: ", false);
+		have_passwd = true;
+		passwdbuf = pgut_malloc(BUFSIZE);
+		memcpy(passwdbuf, buf, sizeof(char)*BUFSIZE);
+	}
 #endif
 
 	if (buf == NULL)
@@ -548,8 +562,7 @@ pgut_connect(const char *info, YesNo prompt, int elevel)
 
 		ereport(elevel,
 			(errcode(E_PG_CONNECT),
-			 errmsg("could not connect to database with \"%s\": %s",
-				info, PQerrorMessage(conn))));
+			 errmsg("could not connect to database: %s", PQerrorMessage(conn))));
 		PQfinish(conn);
 		return NULL;
 	}
